@@ -22,53 +22,47 @@
 #include <string.h>
 #include <math.h>
 #include <unistd.h>
+
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
-#include <X11/extensions/Xfixes.h>
 #include <cspi/spi.h>
-#include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-ui-init.h>
 
 #include "mt-common.h"
-#include "mt-dbus.h"
-#include "mt-main.h"
-#include "mt-session.h"
+#include "mt-service.h"
 #include "mt-pidfile.h"
 #include "mt-ctw.h"
 #include "mt-timer.h"
-
-static AccessibleDeviceListener *button_listener = NULL;
-static AccessibleEventListener  *motion_listener = NULL;
-
-static int fixes_event_base = 0;
+#include "mt-cursor-manager.h"
+#include "mt-cursor.h"
+#include "mt-main.h"
 
 static void
 mt_cursor_set (GdkCursorType type)
 {
     GdkCursor *cursor;
-    GdkWindow *root;
 
     cursor = gdk_cursor_new (type);
-    root = gdk_get_default_root_window ();
-    gdk_window_set_cursor (root, cursor);
+    gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
     gdk_cursor_unref (cursor);
 }
 
 static void
 dwell_restore_single_click (MTClosure *mt)
 {
-    if (mt->dwell_mode == DWELL_MODE_CTW) {
-	mt_ctw_set_click_type (DWELL_CLICK_TYPE_SINGLE);
-	mt_dbus_send_signal (mt->conn, RESTORE_SIGNAL, 0);
-    }
-    else
-	mt->dwell_cct = DWELL_CLICK_TYPE_SINGLE;
+    if (mt->dwell_mode == DWELL_MODE_CTW)
+	mt_ctw_set_clicktype (DWELL_CLICK_TYPE_SINGLE);
+
+    mt_service_set_clicktype (mt->service, DWELL_CLICK_TYPE_SINGLE, NULL);
 }
 
 static void
 dwell_do_pointer_click (MTClosure *mt, gint x, gint y)
 {
-    switch (mt->dwell_cct) {
+    guint clicktype;
+
+    clicktype = mt_service_get_clicktype (mt->service);
+
+    switch (clicktype) {
     case DWELL_CLICK_TYPE_SINGLE:
 	SPI_generateMouseEvent (x, y, "b1c");
 	break;
@@ -137,13 +131,36 @@ analyze_direction (MTClosure *mt, gint x, gint y)
     /* get click type for direction */
     for (i = 0; i < N_CLICK_TYPES; i++)
 	if (mt->dwell_dirs[i] == gd) {
-	    mt->dwell_cct = i;
+	    mt_service_set_clicktype (mt->service, i, NULL);
 	    return TRUE;
 	}
 
     return FALSE;
 }
 
+static void
+draw_line (gint x1, gint y1, gint x2, gint y2)
+{
+    GdkScreen *screen;
+    GdkWindow *root;
+    GdkGC *gc;
+
+    screen = gdk_display_get_default_screen (gdk_display_get_default ());
+    root = gdk_screen_get_root_window (screen);
+
+    gc = gdk_gc_new (root);
+    gdk_gc_set_subwindow (gc, GDK_INCLUDE_INFERIORS);
+    gdk_gc_set_function (gc, GDK_INVERT);
+    gdk_gc_set_line_attributes (gc, 1,
+				GDK_LINE_SOLID,
+				GDK_CAP_ROUND,
+				GDK_JOIN_ROUND);
+    gdk_draw_arc (root, gc, TRUE,
+		  x1 - 4, y1 - 4, 8, 8, 0, 23040);
+    gdk_draw_line (root, gc, x1, y1, x2, y2);
+    g_object_unref (gc);
+}
+
 static void
 dwell_start_gesture (MTClosure *mt)
 {
@@ -174,17 +191,24 @@ dwell_stop_gesture (MTClosure *mt)
     else
 	mt_cursor_set (GDK_LEFT_PTR);
 
+    if (mt->x_old > -1 && mt->y_old > -1) {
+	draw_line (mt->pointer_x, mt->pointer_y, mt->x_old, mt->y_old);
+	mt->x_old = -1;
+	mt->y_old = -1;
+    }
+
     mt->dwell_gesture_started = FALSE;
     mt_timer_stop (mt->dwell_timer);
 }
 
 static void
-dwell_time_elapsed (MtTimer *timer, gpointer data)
+dwell_timer_finished (MtTimer *timer, gpointer data)
 {
     MTClosure *mt = (MTClosure *) data;
     gint x, y;
 
-    gdk_display_get_pointer (gdk_display_get_default(), NULL, &x, &y, NULL);
+    gdk_display_get_pointer (gdk_display_get_default (), NULL, &x, &y, NULL);
+    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
 
     /* stop active drag */
     if (mt->dwell_drag_started) {
@@ -211,17 +235,31 @@ dwell_time_elapsed (MtTimer *timer, gpointer data)
 	else
 	    dwell_start_gesture (mt);
     default:
-	    break;
+	break;
     }
 }
 
-static void
-delay_time_elapsed (MtTimer *timer, gpointer data)
+static gboolean
+right_click_timeout (gpointer data)
 {
     MTClosure *mt = (MTClosure *) data;
 
-    SPI_generateMouseEvent (0, 0, "b1r");
     SPI_generateMouseEvent (mt->pointer_x, mt->pointer_y, "b3c");
+
+    return FALSE;
+}
+
+static void
+delay_timer_finished (MtTimer *timer, gpointer data)
+{
+    MTClosure *mt = (MTClosure *) data;
+
+    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
+
+    SPI_generateMouseEvent (0, 0, "b1r");
+    SPI_generateMouseEvent (mt->pointer_x, mt->pointer_y, "abs");
+
+    g_timeout_add (100, right_click_timeout, data);
 }
 
 /* at-spi callbacks */
@@ -230,143 +268,245 @@ spi_motion_event (const AccessibleEvent *event, void *data)
 {
     MTClosure *mt = (MTClosure *) data;
 
-    if (mt->dwell_enabled)
-	if (!within_tolerance (mt, event->detail1, event->detail2))
-	    if (!mt->dwell_gesture_started) {
-		mt->pointer_x = (gint) event->detail1;
-		mt->pointer_y = (gint) event->detail2;
-		mt_timer_start (mt->dwell_timer);
-	    }
+    if (mt->dwell_enabled) {
+	if (!within_tolerance (mt, event->detail1, event->detail2) &&
+	    !mt->dwell_gesture_started) {
+	    mt->pointer_x = (gint) event->detail1;
+	    mt->pointer_y = (gint) event->detail2;
+	    mt_timer_start (mt->dwell_timer);
+	}
 
-    if (mt_timer_is_running (mt->delay_timer))
-	if (!within_tolerance (mt, event->detail1, event->detail2))
+	if (mt->dwell_gesture_started) {
+	    if (mt->x_old > -1 && mt->y_old > -1)
+		draw_line (mt->pointer_x, mt->pointer_y, mt->x_old, mt->y_old);
+
+	    draw_line (mt->pointer_x, mt->pointer_y,
+		       event->detail1, event->detail2);
+
+	    mt->x_old = event->detail1;
+	    mt->y_old = event->detail2;
+	}
+    }
+
+    if (mt_timer_is_running (mt->delay_timer)) {
+	if (!within_tolerance (mt, event->detail1, event->detail2)) {
 	    mt_timer_stop (mt->delay_timer);
+	    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
+	}
+    }
 }
 
-static SPIBoolean
-spi_button_event (const AccessibleDeviceEvent *event, void *data)
+static void
+spi_button_event (const AccessibleEvent *event, void *data)
 {
     MTClosure *mt = (MTClosure *) data;
 
-    if (event->keycode != 1)
-	return FALSE;
-
-    switch (event->type) {
-    case SPI_BUTTON_PRESSED:
+    if (g_str_equal (event->type, "mouse:button:1p")) {
 	if (mt->delay_enabled) {
 	    gdk_display_get_pointer (gdk_display_get_default (), NULL,
 				     &mt->pointer_x, &mt->pointer_y, NULL);
 	    mt_timer_start (mt->delay_timer);
 	}
+
 	if (mt->dwell_gesture_started)
 	    dwell_stop_gesture (mt);
-	break;
-    case SPI_BUTTON_RELEASED:
-	if (mt_timer_is_running (mt->delay_timer))
-	    mt_timer_stop (mt->delay_timer);
-    default:
-	break;
     }
-
-    return FALSE;
+    else if (g_str_equal (event->type, "mouse:button:1r")) {
+	if (mt->delay_enabled) {
+	    mt_timer_stop (mt->delay_timer);
+	    mt_cursor_manager_restore_all (mt_cursor_manager_get_default ());
+	}
+    }
 }
 
-static GdkFilterReturn
-cursor_changed (GdkXEvent *xevent, GdkEvent *event, gpointer data)
+static gboolean
+cursor_overlay_time (guchar  *image,
+		     gint     width,
+		     gint     height,
+		     MtTimer *timer,
+		     gdouble  time)
 {
-    XEvent *xev = (XEvent *) xevent;
+    cairo_surface_t *surface;
+    cairo_t *cr;
+    gdouble target;
 
-    if (xev->type == fixes_event_base + XFixesCursorNotify) {
-	MTClosure *mt = (MTClosure *) data;
-	XFixesCursorImage *ci;
+    surface = cairo_image_surface_create_for_data (image,
+						   CAIRO_FORMAT_ARGB32,
+						   width, height,
+						   width * 4);
+    if (cairo_surface_status (surface) != CAIRO_STATUS_SUCCESS)
+	return FALSE;
 
-	ci = XFixesGetCursorImage (GDK_DISPLAY());
+    cr = cairo_create (surface);
+    if (cairo_status (cr) != CAIRO_STATUS_SUCCESS)
+	return FALSE;
 
-	if (!mt->dwell_gesture_started)
-	    mt->override_cursor = !g_str_equal (ci->name, "left_ptr");
+    target = mt_timer_get_target (timer);
+    cairo_set_operator (cr, CAIRO_OPERATOR_ATOP);
+    cairo_rectangle (cr, 0, 0, width, height / (target / time));
+    cairo_set_source_rgba (cr, 0., 0., 0., 0.75);
+    cairo_fill (cr);
+    cairo_destroy (cr);
+    cairo_surface_destroy (surface);
 
-	XFree (ci);
-    }
-
-    return GDK_FILTER_CONTINUE;
+    return TRUE;
 }
 
 void
-spi_shutdown (void)
+mt_update_cursor (MtCursor *cursor, MtTimer *timer, gdouble time)
 {
-    SPI_event_quit ();
+    guchar *image;
+    gushort width, height;
+
+    image = mt_cursor_get_image_copy (cursor);
+    if (image == NULL)
+	return;
+
+    mt_cursor_get_dimension (cursor, &width, &height);
+
+    if (cursor_overlay_time (image, width, height, timer, time)) {
+	MtCursorManager *manager;
+	MtCursor *new_cursor;
+	const gchar *name;
+	gushort xhot, yhot;
+
+	name = mt_cursor_get_name (cursor);
+	mt_cursor_get_hotspot (cursor, &xhot, &yhot);
+	new_cursor = mt_cursor_new (name, image, width, height, xhot, yhot);
+	manager = mt_cursor_manager_get_default ();
+	mt_cursor_manager_set_cursor (manager, new_cursor);
+	g_object_unref (new_cursor);
+    }
+
+    g_free (image);
+}
+
+static void
+delay_timer_tick (MtTimer *timer, gdouble time, gpointer data)
+{
+    MTClosure *mt = (MTClosure *) data;
+
+    if (mt->animate_cursor && mt->cursor != NULL)
+	mt_update_cursor (mt->cursor, timer, time);
+}
+
+static void
+dwell_timer_tick (MtTimer *timer, gdouble time, gpointer data)
+{
+    MTClosure *mt = (MTClosure *) data;
+
+    if (mt->animate_cursor && mt->cursor != NULL)
+	mt_update_cursor (mt->cursor, timer, time);
+}
+
+static void
+cursor_cache_cleared (MtCursorManager *manager,
+		      gpointer         data)
+{
+    MTClosure *mt = (MTClosure *) data;
+
+    mt->cursor = mt_cursor_manager_current_cursor (manager);
+}
+
+static void
+cursor_changed (MtCursorManager *manager,
+		const gchar     *name,
+		gpointer         data)
+{
+    MTClosure *mt = (MTClosure *) data;
+
+    if (!mt->dwell_gesture_started)
+	mt->override_cursor = !g_str_equal (name, "left_ptr");
+
+    mt->cursor = mt_cursor_manager_lookup_cursor (manager, name);
 }
 
 static void
 signal_handler (int sig)
 {
-    spi_shutdown ();
+    SPI_event_quit ();
 }
 
 static void
-mt_gconf_notify (GConfClient *client,
-		 const gchar *key,
-		 GConfValue  *value,
-		 gpointer     data)
+gconf_value_changed (GConfClient *client,
+		     const gchar *key,
+		     GConfValue  *value,
+		     gpointer     data)
 {
     MTClosure *mt = (MTClosure *) data;
 
-    if (g_str_equal (key, OPT_THRESHOLD))
+    if (g_str_equal (key, OPT_THRESHOLD) && value->type == GCONF_VALUE_INT)
 	mt->threshold = gconf_value_get_int (value);
-    else if (g_str_equal (key, OPT_DELAY))
+    else if (g_str_equal (key, OPT_DELAY) && value->type == GCONF_VALUE_BOOL)
 	mt->delay_enabled = gconf_value_get_bool (value);
-    else if (g_str_equal (key, OPT_DELAY_T))
-	mt_timer_set_target_time (mt->delay_timer,
-				  gconf_value_get_float (value));
-    else if (g_str_equal (key, OPT_DWELL)) {
+    else if (g_str_equal (key, OPT_DELAY_T) && value->type == GCONF_VALUE_FLOAT)
+	mt_timer_set_target (mt->delay_timer, gconf_value_get_float (value));
+    else if (g_str_equal (key, OPT_DWELL) && value->type == GCONF_VALUE_BOOL) {
 	mt->dwell_enabled = gconf_value_get_bool (value);
 	mt_ctw_update_sensitivity (mt);
 	mt_ctw_update_visibility (mt);
     }
-    else if (g_str_equal (key, OPT_DWELL_T))
-	mt_timer_set_target_time (mt->dwell_timer,
-				  gconf_value_get_float (value));
-    else if (g_str_equal (key, OPT_CTW)) {
+    else if (g_str_equal (key, OPT_DWELL_T) && value->type == GCONF_VALUE_FLOAT)
+	mt_timer_set_target (mt->dwell_timer, gconf_value_get_float (value));
+    else if (g_str_equal (key, OPT_CTW) && value->type == GCONF_VALUE_BOOL) {
 	mt->dwell_show_ctw = gconf_value_get_bool (value);
 	mt_ctw_update_visibility (mt);
     }
-    else if (g_str_equal (key, OPT_MODE)) {
+    else if (g_str_equal (key, OPT_MODE) && value->type == GCONF_VALUE_INT) {
 	mt->dwell_mode = gconf_value_get_int (value);
 	mt_ctw_update_sensitivity (mt);
     }
-    else if (g_str_equal (key, OPT_STYLE)) {
+    else if (g_str_equal (key, OPT_STYLE) && value->type == GCONF_VALUE_INT) {
 	mt->style = gconf_value_get_int (value);
 	mt_ctw_update_style (mt->style);
     }
-    else if (g_str_equal (key, OPT_G_SINGLE))
+    else if (g_str_equal (key, OPT_G_SINGLE) && value->type == GCONF_VALUE_INT)
 	mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE] = gconf_value_get_int (value);
-    else if (g_str_equal (key, OPT_G_DOUBLE))
+    else if (g_str_equal (key, OPT_G_DOUBLE) && value->type == GCONF_VALUE_INT)
 	mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE] = gconf_value_get_int (value);
-    else if (g_str_equal (key, OPT_G_DRAG))
+    else if (g_str_equal (key, OPT_G_DRAG) && value->type == GCONF_VALUE_INT)
 	mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG] = gconf_value_get_int (value);
-    else if (g_str_equal (key, OPT_G_RIGHT))
+    else if (g_str_equal (key, OPT_G_RIGHT) && value->type == GCONF_VALUE_INT)
 	mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT] = gconf_value_get_int (value);
+    else if (g_str_equal (key, OPT_ANIMATE) && value->type == GCONF_VALUE_BOOL) {
+	MtCursorManager *manager;
+
+	manager = mt_cursor_manager_get_default ();
+	mt->animate_cursor = gconf_value_get_bool (value);
+
+	if (mt->animate_cursor)
+	    mt->cursor = mt_cursor_manager_current_cursor (manager);
+	else
+	    mt_cursor_manager_restore_all (manager);
+    }
 }
 
 static void
-mt_gconf_read_options (MTClosure *mt)
+get_gconf_options (MTClosure *mt)
 {
+    gdouble val;
+
     mt->threshold = gconf_client_get_int (mt->client, OPT_THRESHOLD, NULL);
     mt->delay_enabled = gconf_client_get_bool (mt->client, OPT_DELAY, NULL);
     mt->dwell_enabled = gconf_client_get_bool (mt->client, OPT_DWELL, NULL);
     mt->dwell_show_ctw = gconf_client_get_bool (mt->client, OPT_CTW, NULL);
     mt->dwell_mode = gconf_client_get_int (mt->client, OPT_MODE, NULL);
     mt->style = gconf_client_get_int (mt->client, OPT_STYLE, NULL);
+    mt->animate_cursor = gconf_client_get_bool (mt->client, OPT_ANIMATE, NULL);
 
-    mt_timer_set_target_time (mt->delay_timer,
-			      gconf_client_get_float (mt->client, OPT_DELAY_T, NULL));
-    mt_timer_set_target_time (mt->dwell_timer,
-			      gconf_client_get_float (mt->client, OPT_DWELL_T, NULL));
+    val = gconf_client_get_float (mt->client, OPT_DELAY_T, NULL);
+    mt_timer_set_target (mt->delay_timer, val);
+    val = gconf_client_get_float (mt->client, OPT_DWELL_T, NULL);
+    mt_timer_set_target (mt->dwell_timer, val);
 
-    mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE] = gconf_client_get_int (mt->client, OPT_G_SINGLE, NULL);
-    mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE] = gconf_client_get_int (mt->client, OPT_G_DOUBLE, NULL);
-    mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG] = gconf_client_get_int (mt->client, OPT_G_DRAG, NULL);
-    mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT] = gconf_client_get_int (mt->client, OPT_G_RIGHT, NULL);
+    mt->dwell_dirs[DWELL_CLICK_TYPE_SINGLE] =
+	gconf_client_get_int (mt->client, OPT_G_SINGLE, NULL);
+    mt->dwell_dirs[DWELL_CLICK_TYPE_DOUBLE] =
+	gconf_client_get_int (mt->client, OPT_G_DOUBLE, NULL);
+    mt->dwell_dirs[DWELL_CLICK_TYPE_DRAG] =
+	gconf_client_get_int (mt->client, OPT_G_DRAG, NULL);
+    mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT] =
+	gconf_client_get_int (mt->client, OPT_G_RIGHT, NULL);
 }
 
 static MTClosure *
@@ -374,20 +514,33 @@ mt_closure_init (void)
 {
     MTClosure *mt;
 
-    mt = g_try_new0 (MTClosure, 1);
+    mt = g_slice_new0 (MTClosure);
     if (!mt)
 	return NULL;
 
     mt->client = gconf_client_get_default ();
-    mt->dwell_cct = DWELL_CLICK_TYPE_SINGLE;
+    gconf_client_add_dir (mt->client, MT_GCONF_HOME,
+			  GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+    g_signal_connect (mt->client, "value_changed",
+		      G_CALLBACK (gconf_value_changed), mt);
 
     mt->delay_timer = mt_timer_new ();
     g_signal_connect (mt->delay_timer, "finished",
-		      G_CALLBACK (delay_time_elapsed), mt);
+		      G_CALLBACK (delay_timer_finished), mt);
+    g_signal_connect (mt->delay_timer, "tick",
+		      G_CALLBACK (delay_timer_tick), mt);
 
     mt->dwell_timer = mt_timer_new ();
     g_signal_connect (mt->dwell_timer, "finished",
-		      G_CALLBACK (dwell_time_elapsed), mt);
+		      G_CALLBACK (dwell_timer_finished), mt);
+    g_signal_connect (mt->dwell_timer, "tick",
+		      G_CALLBACK (dwell_timer_tick), mt);
+
+    mt->service = mt_service_get_default ();
+    mt_service_set_clicktype (mt->service, DWELL_CLICK_TYPE_SINGLE, NULL);
+
+    mt->x_old = -1;
+    mt->y_old = -1;
 
     return mt;
 }
@@ -397,16 +550,18 @@ mt_closure_free (MTClosure *mt)
 {
     g_object_unref (mt->delay_timer);
     g_object_unref (mt->dwell_timer);
+    g_object_unref (mt->service);
     g_object_unref (mt->client);
-    g_free (mt);
+
+    g_slice_free (MTClosure, mt);
 }
 
 int
 main (int argc, char **argv)
 {
-    GnomeProgram *program;
     MTClosure *mt;
-    int fixes_error_base;
+    MtCursorManager *manager;
+    AccessibleEventListener *bl, *ml;
     pid_t pid;
     gboolean shutdown = FALSE, ctw = FALSE;
     gchar *mode = NULL, *enable = NULL;
@@ -424,11 +579,15 @@ main (int argc, char **argv)
 	{"ctw-y", 'y', 0, G_OPTION_ARG_INT, &pos_y,
 	    _("Window y position"), 0},
 	{"shutdown", 's', 0, G_OPTION_ARG_NONE, &shutdown,
-	    _("Shutdown mousetweaks"), 0},
+	    _("Shut down mousetweaks"), 0},
 	{NULL}
     };
 
-    context = g_option_context_new ("");
+    bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+    bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+    textdomain (GETTEXT_PACKAGE);
+
+    context = g_option_context_new (_("- GNOME mousetweaks daemon"));
     g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
     g_option_context_add_group (context, gtk_get_option_group (FALSE));
     g_option_context_parse (context, &argc, &argv, NULL);
@@ -446,16 +605,19 @@ main (int argc, char **argv)
     }
 
     if ((pid = mt_pidfile_is_running()) >= 0) {
-	g_print ("Already running on PID file %u.\n", pid);
+	g_print ("Daemon is already running. (PID %u)\n", pid);
 	return 1;
     }
 
     g_print ("Starting daemon.\n");
 
-    if ((pid = fork ()) < 0)
+    if ((pid = fork ()) < 0) {
+	g_print ("Fork failed.\n");
 	return 1;
-    else if (pid)
+    }
+    else if (pid) {
 	return 0;
+    }
     else {
 	if (mt_pidfile_create () < 0)
 	    goto FINISH;
@@ -465,22 +627,13 @@ main (int argc, char **argv)
 	signal (SIGQUIT, signal_handler);
 	signal (SIGHUP, signal_handler);
 
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
-
-	program = gnome_program_init (PACKAGE, VERSION,
-				      LIBGNOMEUI_MODULE,
-				      argc, argv,
-				      GNOME_PARAM_NONE);
+	gtk_init (&argc, &argv);
 
 	mt = mt_closure_init ();
-	if (!mt) {
-	    g_object_unref (program);
+	if (!mt)
 	    goto FINISH;
-	}
 
-	if (SPI_init ()) {
+	if (SPI_init () != 0) {
 	    mt_show_dialog (_("Assistive Technologies not enabled"),
 			    _("Mousetweaks requires Assistive Technologies."),
 			    GTK_MESSAGE_ERROR);
@@ -490,40 +643,15 @@ main (int argc, char **argv)
 	    gconf_client_set_bool (mt->client, OPT_DWELL, FALSE, NULL);
 
 	    mt_closure_free (mt);
-	    g_object_unref (program);
-
 	    goto FINISH;
 	}
 
-	mt_session_init ();
-
-	/* listen for cursor changes */
-	if (XFixesQueryExtension (GDK_DISPLAY(),
-				  &fixes_event_base,
-				  &fixes_error_base)) {
-	    XFixesSelectCursorInput (GDK_DISPLAY(),
-				     GDK_ROOT_WINDOW(),
-				     XFixesDisplayCursorNotifyMask);
-	    gdk_window_add_filter (gdk_get_default_root_window (),
-				   cursor_changed,
-				   (gpointer) mt);
-	}
-
 	/* add at-spi listeners */
-	motion_listener = SPI_createAccessibleEventListener
-	    (spi_motion_event, (void *) mt);
-	SPI_registerGlobalEventListener (motion_listener, "mouse:abs");
-
-	button_listener = SPI_createAccessibleDeviceListener
-	    (spi_button_event, (void *) mt);
-	SPI_registerDeviceEventListener (button_listener,
-					 SPI_BUTTON_PRESSED |
-					 SPI_BUTTON_RELEASED,
-					 NULL);
-
-	mt->conn = mt_dbus_init (mt);
-	if (!mt->conn)
-	    goto CLEANUP;
+	ml = SPI_createAccessibleEventListener (spi_motion_event, (void *) mt);
+	SPI_registerGlobalEventListener (ml, "mouse:abs");
+	bl = SPI_createAccessibleEventListener (spi_button_event, (void *) mt);
+	SPI_registerGlobalEventListener (bl, "mouse:button:1p");
+	SPI_registerGlobalEventListener (bl, "mouse:button:1r");
 
 	/* command-line options */
 	if (enable) {
@@ -536,8 +664,10 @@ main (int argc, char **argv)
 		gconf_client_set_bool (mt->client, OPT_DELAY, TRUE, NULL);
 	    }
 	}
+
 	if (ctw)
 	    gconf_client_set_bool (mt->client, OPT_CTW, TRUE, NULL);
+
 	if (mode) {
 	    if (g_str_equal (mode, "gesture"))
 		gconf_client_set_int (mt->client, OPT_MODE,
@@ -547,35 +677,32 @@ main (int argc, char **argv)
 				      DWELL_MODE_CTW, NULL);
 	}
 
-	/* gconf stuff */
-	mt_gconf_read_options (mt);
-	gconf_client_add_dir (mt->client,
-			      MT_GCONF_HOME,
-			      GCONF_CLIENT_PRELOAD_ONELEVEL,
-			      NULL);
-	g_signal_connect (G_OBJECT(mt->client), "value_changed",
-			  G_CALLBACK(mt_gconf_notify), (gpointer) mt);
+	get_gconf_options (mt);
 
-	if (!mt_ctw_init (mt, pos_x, pos_y)) {
-	    mt_dbus_fini (mt->conn);
+	if (!mt_ctw_init (mt, pos_x, pos_y))
 	    goto CLEANUP;
-	}
 
-	/* tell the dwell click applet we are here */
-	mt_dbus_send_signal (mt->conn, ACTIVE_SIGNAL, 1);
+	manager = mt_cursor_manager_get_default ();
+	g_signal_connect (manager, "cursor_changed",
+			  G_CALLBACK (cursor_changed), mt);
+	g_signal_connect (manager, "cache_cleared",
+			  G_CALLBACK (cursor_cache_cleared), mt);
+
+	mt->cursor = mt_cursor_manager_current_cursor (manager);
 
 	SPI_event_main ();
 
-	mt_dbus_send_signal (mt->conn, ACTIVE_SIGNAL, 0);
-	mt_dbus_fini (mt->conn);
+	mt_cursor_manager_restore_all (manager);
+	g_object_unref (manager);
     }
 
 CLEANUP:
-    AccessibleEventListener_unref (motion_listener);
-    AccessibleDeviceListener_unref (button_listener);
+    SPI_deregisterGlobalEventListenerAll (bl);
+    AccessibleDeviceListener_unref (bl);
+    SPI_deregisterGlobalEventListenerAll (ml);
+    AccessibleEventListener_unref (ml);
     SPI_exit ();
     mt_closure_free (mt);
-    g_object_unref (program);
 
 FINISH:
     mt_pidfile_remove ();
